// ============================================================================
// Project:	Deductive Database
// Filename:	ver.h
// Purpose:	Version Settings
// Last Change:	19.09.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2001-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file  ver.h
 * \brief Some basic version settings
 */

/**
 * \dir   base
 * \brief Basic Definitions and Programming Support
 */

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	VER_INCLUDED
#define VER_INCLUDED

//=============================================================================
// For use in Feature Switch Macros:
//=============================================================================

/**
 * True value for use in feature selection macros.
 */

#define YES 1

/**
 * False value for use in feature selection macros.
 */

#define NO  0

//=============================================================================
// Basic Parameters:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_PROG_NAME:
//-----------------------------------------------------------------------------

/**
 * What is the name of this program?
 */

#define VER_PROG_NAME "ydb"

//-----------------------------------------------------------------------------
// VER_PROMPT:
//-----------------------------------------------------------------------------

/**
 * Prompt for input from keyboard (standard input).
 */

#define VER_PROMPT "ydb> "

//-----------------------------------------------------------------------------
// VER_CGI:
//-----------------------------------------------------------------------------

/**
 * Should this program run as CGI program?
 */

#define VER_CGI NO

//-----------------------------------------------------------------------------
// VER_ALERT_FILE:
//-----------------------------------------------------------------------------

/**
 * Name of the file for error messages and warnings.
 */

#define VER_ALERT_FILE "alert"


//=============================================================================
// Settings for Debugging:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_DEBUG:
//-----------------------------------------------------------------------------

/**
 * Check runtime assertions etc. (Debug Version).
 */

// If VER_DEBUG is defined, it should be defined as YES:
// Note that VER_DEBUG cannot be set to NO (0) on the command line.
// It can only be defined, which means YES.
// Undefined usually means NO, but see automatic choice for Visual C++ below!
#ifdef VER_DEBUG
#undef VER_DEBUG
#define VER_DEBUG YES
#endif

// Normally, it will be defined on the command line.
// If one wants to set it here, this would be the place:
#define VER_DEBUG NO
//#define VER_DEBUG YES

// Automatic choice for Visual C++:
#ifndef VER_DEBUG
#ifdef _DEBUG
#define VER_DEBUG YES
#endif
#endif

// If VER_DEBUG is not defined, it should be defined as NO:
#ifndef VER_DEBUG
#define VER_DEBUG NO
#endif

//-----------------------------------------------------------------------------
// VER_FULLCHECK:
//-----------------------------------------------------------------------------

/**
 * Do also time-intensive integrity checks of data structures.
 *
 * Note: This has only an effect if VER_DEBUG is also defined.
 * Otherwise all integrity checks are eliminated.
 */

#define VER_FULLCHECK NO


//-----------------------------------------------------------------------------
// VER_DUMP:
//-----------------------------------------------------------------------------

/**
 * Compile code for data structure dumps (for debugging/testing).
 */

#define VER_DUMP YES

//-----------------------------------------------------------------------------
// VER_DEBUG_OUT_ENABLED:
//-----------------------------------------------------------------------------

/**
 * Compile code for debug output (other than dumps).
 *
 * Note that commenting out this symbol does not necessarily switch off
 * all debug output. Debug output is also controlled by the debug flag
 * in out_c. However, in benchmarks, when checking for this flag costs time,
 * this is compiled in only if this symbol is defined.
 * So comment out this symbol if doing benchmarks.
 */

#define VER_DEBUG_OUT_ENABLED NO


//=============================================================================
// Further Sizes and Limits:
//=============================================================================


//-----------------------------------------------------------------------------
// VER_PAGESIZE:
//-----------------------------------------------------------------------------

/**
 * Size of a memory page in bytes.
 */

#define VER_PAGESIZE 4096
// #define VER_PAGESIZE 8192


//-----------------------------------------------------------------------------
// VER_MAX_TIMERS:
//-----------------------------------------------------------------------------

/**
 * Number of \c timer_c objects existing at the same time.
 */

#define VER_MAX_TIMERS 5


//-----------------------------------------------------------------------------
// VER_MAX_ATOMSIZE:
//-----------------------------------------------------------------------------

/**
 * Maximal length of atoms / identifiers.
 */

#define VER_MAX_ATOMSIZE 20


//-----------------------------------------------------------------------------
// VER_MAX_PRED_ARGS:
//-----------------------------------------------------------------------------

/**
 * Maximal number of arguments of an EDB predicate.
 */

#define VER_MAX_PRED_ARGS 10


//=============================================================================
// List of Benchmarks to be Compiled into the Program:
//=============================================================================

// Please note that this distribution comes only with the TCFF benchmarks.
// Older versions of the other benchmarks can be downloaded from
// http://www.informatik.uni-halle.de/~brass/push/
// Some may require old data structures for relations,
// which were also eliminated from this distribution.


//-----------------------------------------------------------------------------
// Benchmarks of Loaders:
//-----------------------------------------------------------------------------


/**
 * Compile benchmark of handcrafted Loader for DBLP Benchmark.
 */

#define VER_BENCH_DBLP_LOAD NO


/**
 * Compile benchmark of handcrafted Loader for TCFF Benchmark.
 */

#define VER_BENCH_TCFF_LOAD_1 YES


/**
 * Compile benchmark of general Loader for TCFF Benchmark.
 */

#define VER_BENCH_TCFF_LOAD_2 YES


//-----------------------------------------------------------------------------
// DBLP Benchmark:
//-----------------------------------------------------------------------------

/**
 * Compile Ver.1 of DBLP benchmark (Handwritten loader, RelLocalVars).
 */

#define VER_BENCH_DBLP_1 NO

/**
 * Compile Ver.2 of DBLP benchmark (Cursor Interface, Rel+Var in Attributes).
 */

#define VER_BENCH_DBLP_2 NO


/**
 * Compile Ver.3 of DBLP benchmark (Separate Objects).
 */

#define VER_BENCH_DBLP_3 NO


//-----------------------------------------------------------------------------
// Join 1 Benchmark with Query a(X,Y):
//-----------------------------------------------------------------------------


/**
 * Compile Ver.1 of the Join1 a(X,Y) benchmark (RelAttr, no goto, fullDupCh).
 */

#define VER_BENCH_J1AXY_1 YES


/**
 * Compile Ver.2 of the Join1 a(X,Y) benchmark (no goto, bitmap-dupcheck).
 */

#define VER_BENCH_J1AXY_2 YES


/**
 * Compile Ver.3 of the Join1 a(X,Y) benchmark (goto, bitmap-dupcheck, cursor).
 */

#define VER_BENCH_J1AXY_3 YES


/**
 * Compile Ver.4 of the Join1 a(X,Y) benchmark (functions, bitmap-dupcheck).
 */

#define VER_BENCH_J1AXY_4 YES


/**
 * Compile Ver.5 of the Join1 a(X,Y) benchmark (fun without args, bitmap).
 */

#define VER_BENCH_J1AXY_5 YES


/**
 * Compile Ver.6 of the Join1 a(X,Y) benchmark (abstract machine, bitmap).
 */

#define VER_BENCH_J1AXY_6 YES


//-----------------------------------------------------------------------------
// Transitive Closure Benchmark called with both arguments free (TCFF):
//-----------------------------------------------------------------------------

/**
 * Compile Ver.1 of the TCFF benchmark (local vars for relations).
 */

#define VER_BENCH_TCFF_1 YES


/**
 * Compile Ver.2 of the TCFF benchmark (with cursor interface).
 */

#define VER_BENCH_TCFF_2 YES


/**
 * Compile Ver.3 of the TCFF benchmark (seminaive evaluation).
 */

#define VER_BENCH_TCFF_3 YES


/**
 * Compile Ver.4 of the TCFF benchmark (with functions).
 */

#define VER_BENCH_TCFF_4 YES

/**
 * Compile Ver.5 of the TCFF benchmark (with abstract machine).
 */

#define VER_BENCH_TCFF_5 YES


//-----------------------------------------------------------------------------
// Transitive Closure Benchmark called with given Value for first arg. (TCBF):
//-----------------------------------------------------------------------------

/**
 * Compile Ver.1 of the TCBF benchmark (local vars for relations).
 */

#define VER_BENCH_TCBF_1 NO


/**
 * Compile Ver.2 of the TCBF benchmark (cursor interface, RelAttr).
 */

#define VER_BENCH_TCBF_2 NO


/**
 * Compile Ver.4 of the TCFF benchmark (with functions).
 */

#define VER_BENCH_TCBF_4 NO


//-----------------------------------------------------------------------------
// Wine Ontology Benchmark:
//-----------------------------------------------------------------------------

/**
 * Compile the wine ontology benchmark.
 *
 * Please note that this is a very long procedure.
 * Compiling with g++ with -O2 is possible (at least on a machine with big
 * memory), but takes a long time (about 10 minutes on a fast computer).
 * Compiling with "Microsoft Visual Studio Community 2015" on a machine
 * with 4 GB memory gives the error:
 * "fatal error C1002: compiler is out of heap space in pass 2"
 */

#define VER_BENCH_WINE NO


//-----------------------------------------------------------------------------
// Which Handcrafted Loaders are Needed (Old Technology, Only for Comparison)?
//-----------------------------------------------------------------------------

/**
 * Is the old handcrafted loader for the TCFF benchmark needed?
 *
 * This is automatically defined based on the selected benchmarks.
 */

#if VER_BENCH_TCFF_LOAD_1 || VER_BENCH_TCFF_1
#define VER_TCFF_LOAD YES
#else
#define VER_TCFF_LOAD NO
#endif


//=============================================================================
// Data Files For Benchmarks:
//=============================================================================


//-----------------------------------------------------------------------------
// Data File for the DBLP Benchmark:
//-----------------------------------------------------------------------------

/**
 * Data file for the DBLP benchmark (small test version).
 */

#define VER_FILE_DBLP_TEST	"../data/dblp_test.data"


/**
 * Data file for the DBLP benchmark (official version from OpenRuleBench).
 *
 * The file size is 125.434 KB (~122 MB).
 * It contains data about 200.000 publications.
 * The total number of facts (table rows) in the data file is about 2.4 Million
 * (2&nbsp;437&nbsp;867).
 */

#define VER_FILE_DBLP		"../data/dblp.data"


/**
 * Data file for the DBLP benchmark (large version with all data from Oct 2016).
 */

#define VER_FILE_DBLP_BIG	"../data/dblp_big.data"

//-----------------------------------------------------------------------------
// Data Files for the Join1 Benchmarks:
//-----------------------------------------------------------------------------

/**
 * Data file for the Join 1 benchmark (small test file).
 */

#define VER_FILE_JOIN1_TEST	"../data/d1000_test.P"


/**
 * Data file for the Join 1 benchmark (10K rows per table).
 */

#define VER_FILE_JOIN1_10K	"../data/d1000_relsize10000_xsb_cyc.P"


/**
 * Data file for the Join 1 benchmark (50K rows per table).
 */

#define VER_FILE_JOIN1_50K	"../data/d1000_relsize50000_xsb_cyc.P"



/**
 * Data file for the Join 1 benchmark (250K rows per table).
 */

#define VER_FILE_JOIN1_250K	"../data/d1000_relsize250000_xsb_cyc.P"


//-----------------------------------------------------------------------------
// Data Files for the Transitive Closure Benchmarks:
//-----------------------------------------------------------------------------

/**
 * Data file for transitive closure benchmark (small test file).
 */

#define VER_FILE_TC_TEST	"../data/tc_test.P"


/**
 * Data file for transitive closure benchmark (1K nodes, 50K edges).
 */

#define VER_FILE_TC_50K		"../data/tc_d1000_parsize50000_xsb_nocyc.P"


/**
 * Data file for transitive closure benchmark (1K nodes, 50K edges).
 */

#define VER_FILE_TC_50K_CYC	"../data/tc_d1000_parsize50000_xsb_cyc.P"


/**
 * Data file for transitive closure benchmark (1K nodes, 250K edges).
 */

#define VER_FILE_TC_250K	"../data/tc_d1000_parsize250000_xsb_nocyc.P"


/**
 * Data file for transitive closure benchmark (1K nodes, 250K edges).
 */

#define VER_FILE_TC_250K_CYC	"../data/tc_d1000_parsize250000_xsb_cyc.P"


/**
 * Data file for transitive closure benchmark (1K nodes, 500K edges).
 */

#define VER_FILE_TC_500K	"../data/tc_d1000_parsize500000_xsb_nocyc.P"


/**
 * Data file for transitive closure benchmark (1K nodes, 500K edges).
 */

#define VER_FILE_TC_500K_CYC	"../data/tc_d1000_parsize500000_xsb_cyc.P"


/**
 * Data file for transitive closure benchmark (2K nodes, 500K edges).
 */

#define	VER_FILE_TC_500K_D2	"../data/tc_d2000_parsize500000_xsb_nocyc.P"


/**
 * Data file for transitive closure benchmark (2K nodes, 500K edges).
 */

#define VER_FILE_TC_500K_D2_CYC	"../data/tc_d2000_parsize500000_xsb_cyc.P"


/**
 * Data file for transitive closure benchmark (2K nodes, 1M edges).
 */

#define VER_FILE_TC_1M_D2	"../data/tc_d2000_parsize1000000_xsb_nocyc.P"


/**
 * Data file for transitive closure benchmark (2K nodes, 1M edges).
 */

#define VER_FILE_TC_1M_D2_CYC	"../data/tc_d2000_parsize1000000_xsb_cyc.P"


//-----------------------------------------------------------------------------
// Data File for the Wine Ontology Benchmark:
//-----------------------------------------------------------------------------

/**
 * Data File for the Wine Ontology Benchmark.
 */

#define VER_FILE_WINE	"../data/wine_data.P"


//=============================================================================
// Select an Operating System:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_UNIX/VER_WINDOWS: Select operating system
//-----------------------------------------------------------------------------

/**
 * \def   VER_UNIX
 * Is the program being compiled on a UNIX-like system?
 *
 * \def   VER_WINDOWS
 * Is the program being compiled on a Windows system?
 */


// Make sure that if VER_UNIX is defined on the commandline, it is YES:
#ifdef VER_UNIX
#undef VER_UNIX
#define VER_UNIX YES
#endif

// Make sure that if VER_WINDOWS is defined on the commandline, it is YES:
#ifdef VER_WINDOWS
#undef VER_WINDOWS
#define VER_WINDOWS YES
#endif


// Select here which version you want if the makefile does not define it:
#ifndef VER_UNIX
#ifndef VER_WINDOWS
// #define VER_WINDOWS YES
// #define VER_UNIX YES
// If none defined, we choose automatically below.
#endif // VER_WINDOWS
#endif // VER_UNIX


// Still not defined? Check for typical Microsoft Symbols, else choose UNIX:
#ifndef VER_UNIX
#ifndef VER_WINDOWS

#if defined(_MSC_VER) || defined(_WINDOWS_) || defined(WIN32)
#define VER_WINDOWS YES
#else
#define VER_UNIX YES
#endif

#endif // VER_WINDOWS
#endif // VER_UNIX


//=============================================================================
// Portability Definitions:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_CS_ISO:
//-----------------------------------------------------------------------------

/**
 * Allow national characters according to ISO Latin 8859/1 code.
 */

#if VER_UNIX
#define VER_CS_ISO YES
#else
#define VER_CS_ISO NO
#endif

//-----------------------------------------------------------------------------
// VER_CS_MSDOS:
//-----------------------------------------------------------------------------

/**
 * Allow national characters according to some MSDOS code.
 *
 * Only one of VER_CS_ISO and VER_CS_MSDOS may be defined.
 * If none is defined, only ASCII characters are accepted.
 */

#if VER_WINDOWS
#define VER_CS_MSDOS YES
#else
#define VER_CS_MSDOS NO
#endif


//-----------------------------------------------------------------------------
// VER_IOSTREAM:
//-----------------------------------------------------------------------------

/**
 * Use the iostream library (else: low level UNIX functions).
 */

#if VER_UNIX
#define VER_IOSTREAM NO
#else
#define VER_IOSTREAM YES
#endif

//-----------------------------------------------------------------------------
// Switch off Warning C4514: unreferenced inline function has been removed
//-----------------------------------------------------------------------------

// Microsoft Visual C++ 6.0 generates this warning in the highest warning
// level 4 when an include file defines an inline function not used in this
// source file. However, it might be used in a different source file which
// includes the same include file.

// #if	VER_WINDOWS
// #pragma warning (disable: 4514)
// #endif

//-----------------------------------------------------------------------------
// VER_UNUSED:
//-----------------------------------------------------------------------------

/**
 * Mark unused arguments so that the warning is not printed.
 */

#ifdef __GNUC__
#define VER_UNUSED(ARG)	ARG __attribute__((unused))
#else
#define VER_UNUSED(ARG) ARG
#endif

//-----------------------------------------------------------------------------
// VER_PRESS_RETURN:
//-----------------------------------------------------------------------------

/**
 * Do we need to wait at the end before the program ends?
 */

#if VER_WINDOWS
#define VER_PRESS_RETURN YES
#else
#define VER_PRESS_RETURN NO
#endif


//-----------------------------------------------------------------------------
// VER_SIGNAL:
//-----------------------------------------------------------------------------

/**
 * Try to catch signals.
 */

#if	VER_UNIX
#define VER_SIGNAL YES
#else
#define VER_SIGNAL NO
#endif


//-----------------------------------------------------------------------------
// VER_BYTE:
//-----------------------------------------------------------------------------

/**
 * \typedef VER_BYTE
 * Data type for bytes in data blocks (sizeof must be 1).
 */

typedef char VER_BYTE;

//-----------------------------------------------------------------------------
// VER_USHORT:
//-----------------------------------------------------------------------------

/**
 * Data type for 16 Bit numbers (unsigned), sizeof must be 2.
 */

typedef unsigned short VER_USHORT;

//-----------------------------------------------------------------------------
// VER_INT:
//-----------------------------------------------------------------------------

/**
 * Data type for 32 Bit numbers, sizeof must be 4.
 */

typedef int VER_INT;

//-----------------------------------------------------------------------------
// VER_UINT:
//-----------------------------------------------------------------------------

/**
 * Data type for unsigned 32 Bit numbers, sizeof must be 4.
 */

typedef unsigned int VER_UINT;

//-----------------------------------------------------------------------------
// VER_UINT_BITS:
//-----------------------------------------------------------------------------

/**
 * Number of bits in an unsigned int (used for bitmaps).
 */

#define VER_UINT_BITS	32

//-----------------------------------------------------------------------------
// VER_INT64:
//-----------------------------------------------------------------------------

/**
 * Data type for 64 Bit numbers (if compiled on 64 Bit machine).
 *
 *Must be sufficient to hold a pointer.
 */


// gcc -pedantic complains:
// ver.h:240:14: warning: ISO C++ 1998 does not support "long long"

#ifdef __GNUC__
#include <stdint.h>
typedef int64_t VER_INT64;
#else
typedef long long VER_INT64;
#endif

//-----------------------------------------------------------------------------
// VER_UINT64:
//-----------------------------------------------------------------------------

/**
 * Data type for 64 Bit numbers (if compiled on 64 Bit machine).
 *
 * Must be sufficient to hold a size_t value.
 */

//typedef unsigned long long VER_UINT64;

//-----------------------------------------------------------------------------
// VER_SHORTOFFSET: How many shorts are n ints? Answer must be 2*n.
//-----------------------------------------------------------------------------

//#define VER_SHORTOFFSET(N)	((N)*2)
	// The Data part of a block is a union of VER_BYTE, VER_USHORT, VER_INT
	// arrays. Sometimes different types are used in a block, but one must
	// make sure that the same memory location is not used with different
	// types.


//=============================================================================
// End of Include File:
//=============================================================================

#endif

