// ============================================================================
// Project:	Deductive Database
// Filename:	inst.h
// Purpose:	Enumeration Type for Abstract Machine Instructions (OpCodes)
// Last Change:	19.09.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file
 * \brief Enumeration type for Abstract Machine Instructions (OpCodes)
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	INST_INCLUDED
#define INST_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#ifndef STR_INCLUDED
#include "../base/str.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif

#ifndef ITYPE_INCLUDED
#include "itype.h"
#endif

//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// inst_t: Instructions of the bottom-up abstract machine (OpCodes).
//-----------------------------------------------------------------------------

/**
 * Instructions of the bottom-up abstract machine (OpCodes).
 */

typedef enum inst_enum {
	// Null instruction (NOOP):
	INST_NULL,

	// Instruction to stop program execution:
	INST_HALT,

	// Procedure calls:
	INST_CALL,
	INST_RETURN,

	// Duplicate check:
	INST_DUPCHECK,
	INST_DUPCHECK_2,
	INST_DUPCHECK_2B,

	// Saving and restoring values of variables:
	INST_SAVE_VAR,
	INST_RESTORE_VAR,

	// Instructions for saving and restoring cursors:
	INST_SAVE_LIST_CUR,
	INST_SAVE_MMAP_CUR,
	INST_SAVE_MMAP_1_1_CUR,
	INST_RESTORE_LIST_CUR,
	INST_RESTORE_MMAP_CUR,
	INST_RESTORE_MMAP_1_1_CUR,

	// Instructions for loops over cursors:
	INST_LOOP_LIST,
	INST_LOOP_LIST_2,
	INST_LOOP_MMAP,
	INST_LOOP_MMAP_1_1,
	INST_GET_COL,
	INST_GET_LIST_2_COL_1,
	INST_GET_LIST_2_COL_2,
	INST_GET_MMAP_1_1_OUT_1,
	INST_END_LOOP,
	INST_END_LOOP_LIST_2,
	INST_END_LOOP_MMAP_1_1,

	// Conditions:
	INST_IF_VAR_IS,
	INST_IF_VAR_EQ,

	// Inserting tuples in relations:
	INST_INSERT_LIST,
	INST_INSERT_LIST_2,
	INST_INSERT_LIST_2_V,
	INST_INSERT_MMAP,
	INST_INSERT_MMAP_1_1,

	// Debugging support:
	INST_TRACE_CALL_2
} inst_t;

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// inst_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#if	VER_DEBUG

	/**
	 * This checks whether a given value is indeed a valid member of the
	 * enumeration type.
	 */

bool	inst_valid(inst_t inst);
#endif

//-----------------------------------------------------------------------------
// inst_name:
//-----------------------------------------------------------------------------

	/**
	 * String representation of the instruction OpCode.
	 *
	 * @return OpCode as string, e.g. "CALL".
	 */

str_t	inst_name(inst_t inst);

//-----------------------------------------------------------------------------
// inst_type:
//-----------------------------------------------------------------------------

	/**
	 * Type of the instruction based on number and length of arguments.
	 *
	 * @return value of enumeration type itype_t, e.g. ITYPE_8_16.
	 */

itype_t	inst_type(inst_t inst);


//=============================================================================
// End of File:
//=============================================================================

#endif

