// ============================================================================
// Project:	Deductive Database
// Filename:	itype.h
// Purpose:	Enumeration Type for classifying BAM instructions by arguments
// Last Change:	19.09.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file
 * \brief Enumeration type for classifying BAM instructions by arguments
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	ITYPE_INCLUDED
#define ITYPE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#ifndef STR_INCLUDED
#include "../base/str.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif

//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// itype_t: Instruction types (classified by number and length of arguments).
//-----------------------------------------------------------------------------

/**
 * Instructions types (classified by number and length of arguments).
 */

typedef enum itype_enum {
	// Instruction with no arguments:
	ITYPE_NONE,

	// Instruction with one 8 bit argument:
	ITYPE_8,

	// Instruction with one 16 bit argument:
	ITYPE_16,

	// Instruction with two 8 bit arguments:
	ITYPE_8_8,

	// Instruction with three 8 bit arguments:
	ITYPE_8_8_8,

	// Instruction with one 8 bit argument and one 16 bit argument:
	ITYPE_8_16,

	// Instruction with three arguments (8 bit, 16 bit, 8 bit):
	ITYPE_8_16_8,

} itype_t;


//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// itype_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#if	VER_DEBUG

	/**
	 * This checks whether a given value is indeed a valid member of the
	 * enumeration type.
	 */

bool	itype_valid(itype_t itype);
#endif

//-----------------------------------------------------------------------------
// itype_name: String representation of this instruction type.
//-----------------------------------------------------------------------------

	/**
	 * This returns a string representation of the instruction type.
	 *
	 * @return enumeration value as string, e.g. "ITYPE_8".
	 */

str_t	itype_name(itype_t itype);


//=============================================================================
// End of File:
//=============================================================================

#endif

